package com.shixing.sxedit;


import com.shixing.sxedit.types.SXLogLevel;
import com.shixing.sxedit.util.Size;
import com.shixing.sxedit.util.TextUtils;

/**
 * 剪辑管理类
 *
 * */

public class SXEditManager {

    /**
     * 创建剪辑管理器
     *
     * <p>
     *     1. 创建时 options 将会被复制, 对当前 options 的修改, 不会重新应用到剪辑管理器
     *     2. 使用完成后, 必须调用 destroy 方式释放
     * </p>
     *
     * @param options
     *
     * */
    public SXEditManager(SXEditOptions options) {
        mNativeManager = nCreateEditManager(options.getNativeOptions());
    }

    /**
     * 创建剪辑管理器
     *
     * <p>
     *     1. 创建时 options 将会被复制, 对当前 options 的修改, 不会重新应用到剪辑管理器
     *     2. 使用完成后, 必须调用 destroy 方式释放
     * </p>
     *
     * @param options 配置信息
     * @param json, toJson 导出的剪辑信息
     *
     * */
    public SXEditManager(SXEditOptions options, String json) {
        mNativeManager = nCreateEditManagerFromJson(options.getNativeOptions(), json);
    }

    /**
     * 释放创建的 EditManager
     *
     * */
    public void destroy() {
        if (mNativeManager > 0) {
            nDestroy(mNativeManager);
            mNativeManager = 0;
        }
    }

    /**
     * 设置日志级别, 默认为 SXLogLevel.LOG_INFO
     *
     * @param level, {@link SXLogLevel}
     * */
    public static void setLoggerLevel(SXLogLevel level) {
        nSetLoggerLevel(level.ordinal());
    }

    /**
     * 获取分辨率大小
     *
     * @return Size, {@link Size}
     * */
    public Size getSize() {
        if (mNativeManager > 0) {
            int[] size = nSize(mNativeManager);
            return new Size(size[0], size[1]);
        }
        return new Size(0, 0);
    }

    /**
     * 合成帧率
     * @return 帧率
     */
    public double getFps() {
        if (mNativeManager > 0) {
            return nFps(mNativeManager);
        }
        return 1;
    }


    /**
     * 获取音频管理器
     *
     * @return {@link SXAudioManager}
     * */
    public SXAudioManager getAudioManger() {
        return new SXAudioManager(nAudioManager(mNativeManager));
    }

    /**
     * 通过 groupId 获取对应的组
     *
     * @param groupId 组Id
     * @return {@link SXTrackGroup}
     */
    public SXTrackGroup getGroup(String groupId) {
        if (mNativeManager > 0) {
            long group = nGroup(mNativeManager, groupId);
            if (group > 0) {
                return new SXTrackGroup(group, mNativeManager, SXTrackGroup.SXGroupOwnType.Editor);
            }
        }
        return null;
    }
    /**
     * 添加组到末尾
     *
     * @return {@link SXTrackGroup}
     */
    public SXTrackGroup addNewGroup() {
        if (mNativeManager > 0) {
            long group = nAddNewGroup(mNativeManager);
            if (group > 0) {
                return new SXTrackGroup(group, mNativeManager, SXTrackGroup.SXGroupOwnType.Editor);
            }
        }
        return null;
    }

    /**
     * 在指定位置添加组
     *
     * @return {@link SXTrackGroup}
     * */
    public SXTrackGroup addNewGroupAt(int index) {
        if (mNativeManager > 0) {
            long group = nAddGroupAt(mNativeManager, index);
            if (group > 0) {
                return new SXTrackGroup(group, mNativeManager, SXTrackGroup.SXGroupOwnType.Editor);
            }
        }
        return null;
    }

    /**
     * 通过指定 groupId 删除组
     *
     * @note 只会删除组, 不会删除释放组内的 SXTrack
     * @param  groupId
     * */
    public void deleteGroup(String groupId) {
        if (mNativeManager > 0) {
            nDeleteGroup(mNativeManager, groupId);
        }
    }

    /**
     * 获取所有组
     *
     * @return SXTrackGroup[]
     * */
    public SXTrackGroup[] getGroups() {
        if (mNativeManager > 0) {
            long[] nativeGroups = nGroups(mNativeManager);
            if (nativeGroups != null) {
                SXTrackGroup[] groups = new SXTrackGroup[nativeGroups.length];
                for (int i = 0; i < nativeGroups.length; i++) {
                    groups[i] = new SXTrackGroup(nativeGroups[i], mNativeManager, SXTrackGroup.SXGroupOwnType.Editor);
                }
                return groups;
            }
        }
        return null;
    }

    /**
     * 移动指定组到指定位置
     *
     * @param groupId
     * @param index
     * */
    public void moveGroupTo(String groupId, int index) {
        if (mNativeManager > 0) {
            nMoveGroupTo(mNativeManager, groupId, index);
        }
    }

    /**
     * 通过本地文件路径, 创建媒体轨道，接受视频和图片素材
     *
     * @note 图片轨道时长默认时长为3秒，其他素材使用素材自身时长
     *
     * @param sourcePath 资源本地路径
     * @return {@link SXMediaTrack}
     */
    public SXMediaTrack createMediaTrack(String sourcePath) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(sourcePath)) {
            long track = nCreateMediaTrack(mNativeManager, sourcePath);
            if (track > 0) {
                return new SXMediaTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Editor);
            }
        }
        return null;
    }
    /**
     * 通过 {@link SXResource}, 创建媒体轨道，接受视频和图片素材
     *
     * @note 图片轨道时长默认时长为3秒，其他素材使用素材自身时长
     *
     * @param resource 资源本地路径
     * @return {@link SXMediaTrack}
     */
    public SXMediaTrack createMediaTrackWithResource(SXResource resource) {
        if (mNativeManager > 0) {
            long track = nCreateMediaTrackWithResource(mNativeManager, resource.getNativeResource());
            if (track > 0) {
                return new SXMediaTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Editor);
            }
        }
        return null;
    }
    /**
     * 通过本地目录路径创建媒体轨道
     *
     * @param sourcePath, 资源本地路径
     * @param duration, 显示时长, 单位: 秒
     * @return {@link SXStickerTrack}
     */
    public SXStickerTrack createStickerTrack(String sourcePath, double duration) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(sourcePath)) {
            long track = nCreateStickerTrack(mNativeManager, sourcePath, duration);
            if (track > 0) {
                return new SXStickerTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Editor);
            }
        }
        return null;
    }
    /**
     * 创建文字轨道
     *
     * @param duration, 显示时长, 单位: 秒
     * @return {@link SXTextTrack}
     */
    public SXTextTrack createTextTrack(double duration) {
        if (mNativeManager > 0) {
            long textTrack = nCreateTextTrack(mNativeManager, duration);
            if (textTrack > 0) {
                return new SXTextTrack(textTrack, mNativeManager, SXTrack.SXTrackOwnType.Editor);
            }
        }
        return null;
    }
    /**
     * 通过本地文件路径, 创建媒体轨道，接受视频和音频文件
     *
     * @param sourcePath 资源本地路径
     * @param duration, 持续时长, 单位: 秒
     * @return {@link SXAudioTrack}
     */
    public SXAudioTrack createAudioTrack(String sourcePath, double duration) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(sourcePath)) {
            long track = nCreateAudioTrack(mNativeManager, sourcePath, duration);
            if (track > 0) {
                return new SXAudioTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Editor);
            }
        }
        return null;
    }

    /**
     * 删除指定轨道
     *
     * @param trackId
     * */
    public void deleteTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            nDeleteTrack(mNativeManager, trackId);
        }
    }

    /**
     * 复制轨道
     *
     * @param trackId
     * @return {@link SXTrack}
     * */
    public SXTrack cloneTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            long track = nCloneTrack(mNativeManager, trackId);
            if (track > 0) {
                return SXTrack.createNewTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Editor);
            }
        }
        return null;
    }
    /**
     * 获取轨道
     *
     * @param trackId
     * @return {@link SXTrack}
     * */
    public SXTrack getTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            long track = nTrack(mNativeManager, trackId);
            if (track > 0) {
                return SXTrack.createNewTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Editor);
            }
        }
        return null;
    }


    /**
     * 创建合成
     *
     * @param width, 宽, px
     * @param height, 高, px
     * @param frames, 总帧数
     * @return {@link SXComposite}
     * */
    public SXComposite createComposite(int width, int height, long frames) {
        if (mNativeManager > 0) {
            long composite = nCreateComposite(mNativeManager, width, height, frames);
            if (composite > 0) {
                return new SXComposite(composite, mNativeManager);
            }
        }
        return null;
    }

    /**
     * 通过 id 获取合成
     *
     * @param compositeId
     * @return {@link SXComposite}
     * */
    public SXComposite getComposite(String compositeId) {
        if (mNativeManager > 0) {
            long composite = nComposite(mNativeManager, compositeId);
            if (composite > 0) {
                return new SXComposite(composite, mNativeManager);
            }
        }
        return null;
    }

    /**
     * 通过 id 删除合成
     *
     * @param compositeId
     * */
    public void deleteComposite(String compositeId) {
        if (mNativeManager > 0) {
            nDeleteComposite(mNativeManager, compositeId);
        }
    }

    /**
     * 获取总时长
     *
     * @return double, 时长, 单位: 秒
     * */
    public double getDuration() {
        if (mNativeManager > 0) {
            return nDuration(mNativeManager);
        }
        return 0;
    }

    /**
     * 设置总时长
     *
     * @param duration, 时长, 单位: 秒
     *
     * */
    public void setDuration(double duration) {
        if (mNativeManager > 0) {
            nSetDuration(mNativeManager, duration);
        }
    }

    /**
     * 获取总帧数
     *
     * @return long, 帧数
     * */
    public long getFrames() {
        if (mNativeManager > 0) {
            return nFrames(mNativeManager);
        }
        return 0;
    }

    /**
     * 获取最后一个轨道结束的时间
     *
     * @param excludeAudio, 是否包含音频轨道
     * @return double, 时长, 单位: 秒
     * */
    public double getContentMaxTime(boolean excludeAudio) {
        if (mNativeManager > 0) {
            return nContentMaxTime(mNativeManager, excludeAudio);
        }
        return 0;
    }

    /**
     * 设置总帧数
     *
     * @param frames, 帧数
     * */
    public void setDurationInFrame(long frames) {
        if (mNativeManager > 0) {
            nSetDurationInFrame(mNativeManager, frames);
        }
    }

    /**
     * 设置背景颜色
     *
     * @param color
     * */
    public void setBackgroundColor(int color) {
        if (mNativeManager > 0) {
            float r = ((color >> 16) & 0xff) / 255.0f;
            float g = ((color >>  8) & 0xff) / 255.0f;
            float b = ((color      ) & 0xff) / 255.0f;

            nSetBackgroundColor(mNativeManager, r, g, b);
        }
    }

    /**
     * 设置背景颜色
     *q
     * @param red
     * @param green
     * @param blue
     * */
    public void setBackgroundColor(float red, float green, float blue) {
        if (mNativeManager > 0) {
            nSetBackgroundColor(mNativeManager, red, green, blue);
        }
    }

    /**
     * 设置背景颜色
     *
     * @param hexColorString, 十六进制颜色字符串, 比如 #00FF00, #00FF00FF
     * */
    public void setBackgroundColor(String hexColorString) {
        if (mNativeManager > 0) {
            nSetBackgroundColorByHexString(mNativeManager, hexColorString);
        }
    }


    /***
     * 导出剪辑信息
     *
     * @return  json
     * */
    public String toJson() {
        if (mNativeManager> 0) {
            return nToJson(mNativeManager);
        }
        return "";
    }

    private static final String TAG = "SXEditManager";

    private long mNativeManager;
    public long getNativeManager() {
        return mNativeManager;
    }

//    public SXTrack hitTest(PointF point) {
//        if (mNativeManager > 0) {
//            long track = nHitTest(mNativeManager, point.x, point.y);
//            if (track > 0) {
//                return SXTrack.createNewTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Editor);
//            }
//        }
//        return null;
//    }
//
//    public SXTrack hitTest(PointF point, int type) {
//        if (mNativeManager > 0) {
//            long track = nHitTest(mNativeManager, point.x, point.y, type);
//            if (track > 0) {
//                return SXTrack.createNewTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Editor);
//            }
//        }
//        return null;
//    }


    public static void loadLibrary() {
        System.loadLibrary("SXEditJni");
        nSetLoggerLevel(SXLogLevel.LOG_INFO.ordinal());
        nInitLogger();
        System.out.println("SXEditJni loaded");
    }

    static {
        loadLibrary();
    }

    static native long nCreateEditManager(long nativeOptions);
    static native long nCreateEditManagerFromJson(long nativeOptions, String json);
    static native void nDestroy(long nativeManager);
    static native String nToJson(long nativeManager);
    static native long nAudioManager(long nativeManager);
    static native double nFps(long nativeManager);
    static native int[] nSize(long nativeManager);
    static native double nDuration(long nativeManager);
    static native void nSetDuration(long nativeManager, double duration);
    static native double nContentMaxTime(long nativeManager, boolean exclude_audio);
    static native long nFrames(long nativeManager);
    static native void nSetDurationInFrame(long nativeManager, long frames);
    static native void nSetCurrentTime(long mNativeManager, double time);
    static native boolean nStepForward(long mNativeManager);
    static native void nSetBackgroundColor(long nativeManager, float red, float green, float blue);
    static native void nSetBackgroundColorByHexString(long nativeManager, String hexString);


    static native long nGroup(long nativeManager, String groupId);
    static native long nAddNewGroup(long nativeManager);
    static native long nAddGroupAt(long nativeManager, int index);
    static native void nDeleteGroup(long nativeManager, String groupId);
    static native void nMoveGroupTo(long nativeManager, String groupId, int index);
    static native long[] nGroups(long nativeManager);


    static native long nCreateMediaTrack(long nativeManager, String sourcePath);
    static native long nCreateMediaTrackWithResource(long nativeManager, long nativeSource);
    static native long nCreateStickerTrack(long nativeManager, String sourcePath, double duration);
    static native long nCreateTextTrack(long nativeManager, double duration);
    static native long nCreateAudioTrack(long nativeManager, String sourcePath, double duration);
    static native void nDeleteTrack(long nativeManager, String trackId);
    static native long nCloneTrack(long nativeManager, String trackId);
    static native long nTrack(long nativeManager, String trackId);

    static native long nCreateComposite(long nativeManager, int width, int height, long frames);
    static native long nComposite(long nativeManager, String compositeId);
    static native void nDeleteComposite(long nativeManager, String compositeId);

    static native long nHitTest(long nativeManager, float x, float y);
    static native long nHitTest(long nativeManager, float x, float y, int type);

    static native void nInitLogger();
    static native void nSetLoggerLevel(int level);
}
